SystemOrganization addCategory: #'Rest-Server-Handlers'!
SystemOrganization addCategory: #'Rest-Server-Services'!
SystemOrganization addCategory: #'Rest-Server'!

!HTTPPut methodsFor: '*rest-server' stamp: 'topa 3/3/2009 21:43'!
for: aRequestLine readFrom: aSwazooStream 

	| contentTypeField |
	super for: aRequestLine readFrom: aSwazooStream.

	contentTypeField := self headers fieldOfClass: ContentTypeField
		ifNone: [SwazooHTTPPutError raiseSignal: 'Missing Content-Type'].
	self octetDataFrom: aSwazooStream.! !

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:18'!
process: anHttpRequest
	"Handles an http request. Should answer with an http response."
	
	self subclassResponsibility! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResourceHttpHandler class methodsFor: 'error handling' stamp: 'topa 3/3/2009 21:01'!
responseForFailure

	^ HTTPResponse internalServerError.! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'mt 3/2/2009 14:52'!
pathComponents: anHTTPRequest
	"Return a list with all components of the path in the HTTP request. Remove a leading '/'."
	
	self flag: #TODO. "What about prefixes here?"
	^ anHTTPRequest uri identifierPath allButFirst.! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 3/2/2009 14:43'!
process: anHTTPRequest 
	"Create a resource chain from the path of the http request. Fail if that chain is empty. Redirect processing according to used verb (POST, PUT, GET, DELETE)."
	
	| processorClass |

	anHTTPRequest isGet ifTrue: [processorClass := RestGetRequestProcessor].
	anHTTPRequest isPost ifTrue: [processorClass := RestPostRequestProcessor].
	anHTTPRequest isDelete ifTrue: [processorClass := RestDeleteRequestProcessor].
	anHTTPRequest isPut ifTrue: [processorClass := RestPutRequestProcessor].
	[^ (processorClass newFrom: anHTTPRequest handler: self)
		process: (self rootComponent: anHTTPRequest ifAbsent: [^ self class responseForFailure])
		components: (self pathComponents: anHTTPRequest) allButFirst]
	on: RestResolveError do: [:error | ^ error asHTTPResponse].! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'mt 3/2/2009 15:03'!
rootComponent: anHTTPRequest ifAbsent: aBlock
	"Return an association between the resource name and the resource class for the root component according to the HTTP request."

	^ self class rootResources
		associationAt: ((self pathComponents: anHTTPRequest) at: 1 ifAbsent: aBlock)
		ifAbsent: aBlock! !

Object subclass: #RestRequestProcessor
	instanceVariableNames: 'request handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

RestRequestProcessor subclass: #RestDeleteRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestDeleteRequestProcessor methodsFor: 'processing' stamp: 'topa 3/3/2009 21:00'!
responseBodyForRoot: root components: components
	"Do recursive entity resolving. Start with root. Return an empty body due to DELETE request."
	
	| entity |
	entity := self resolveId: root value all components: components.

	(entity respondsTo: #delete)
		ifTrue: [entity perform: #delete]
		ifFalse: [RestResolveError signal: 'Selector #delete not found.'].
	^ String new! !

RestRequestProcessor subclass: #RestGetRequestProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:13'!
responseBodyForRoot: root components: pathComponents
	"Do recursive entity resolving. Start with root."
		
	| entity |
	entity := self resolveId: root value all components: pathComponents.

	^ Json render: (entity isCollection
		ifTrue: [entity collect: [:each | each restUri]]
		ifFalse: [entity])! !

RestRequestProcessor subclass: #RestPostRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPostRequestProcessor methodsFor: 'adding' stamp: 'mt 3/2/2009 15:50'!
addResource: aResourceName to: anEntity
	"Resolve the selector and perform an add-operation on the given entity with the postData."
	
	| selector |
	selector := self selectorFor: aResourceName.
	
	(anEntity respondsTo: selector)
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
		
	^ (anEntity perform: selector with: self postData) restUri
! !

!RestPostRequestProcessor methodsFor: 'private' stamp: 'mt 3/5/2009 10:40'!
postData
	"Data could be JSON string with properties or a simple string with a rest uri (not JSON). The first one will be parsed into a dictionary. The last one will be resolved to the real entity."

	self flag: #TODO. "data should ALWAYS be JSON parsable. -> '[resturi]' or '{properties}'"
	postData ifNil: [ | data |
		data := self request ensureFullRead entityBody value.
		postData := (data startsWith: '{')
			ifTrue: [Json readFrom: data readStream]
			ifFalse: [self resolveUri: data]].
	^ postData! !

!RestPostRequestProcessor methodsFor: 'resolving' stamp: 'mt 3/2/2009 15:55'!
resolveMethod: anEntity components: pathComponents
	"New recursion abort condition. Perform add-operation due to POST request (instead of returning the entity)."
	
	^ pathComponents size = 1
		ifTrue: [self addResource: pathComponents first to: anEntity]
		ifFalse: [super resolveMethod: anEntity components: pathComponents]! !

!RestPostRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:51'!
responseBodyForRoot: root components: components
	"Do recursive entity resolving. Start with root."
	
	^ components isEmpty
		ifTrue: [self addResource: root key to: root value]
		ifFalse: [self resolveId: root value all components: components]! !

!RestPostRequestProcessor methodsFor: 'adding' stamp: 'mt 3/2/2009 15:45'!
selectorFor: resourceName
	"Return the selector for adding a resource depending on the given resource name."
	
	| selector |
	selector := self postData isDictionary
		ifTrue: ['addNew', resourceName capitalized]
		ifFalse: ['add', resourceName capitalized].
	(selector endsWith: 'ies') ifTrue: [selector := (selector truncateTo: selector size - 3), 'y'].
	(selector endsWith: 's') ifTrue: [selector := selector truncateTo: selector size - 1].
	selector := (selector, ':') asSymbol.
	^ selector! !

RestRequestProcessor subclass: #RestPutRequestProcessor
	instanceVariableNames: 'putData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPutRequestProcessor methodsFor: 'private' stamp: 'mt 3/2/2009 16:00'!
putData

	putData ifNil: [
		putData := Json readFrom: (HTTPString stringFromBytes: self request putData)
			value "convert bytes to string"
			readStream "convert string to readstream"].
	^ putData! !

!RestPutRequestProcessor methodsFor: 'processing' stamp: 'topa 3/3/2009 21:01'!
responseBodyForRoot: root components: pathComponents
	"Do recursive entity resolving. Start with root."
	
	| entity selector |
	entity := self resolveId: root value all components: pathComponents.
	self putData keysAndValuesDo: [:key :value | 
		selector := (key, ':') asSymbol.
		((entity class primitivePropertyNames includes: key asSymbol) and:
			[entity respondsTo: selector])
				ifTrue: [entity perform: selector with: value]
				ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})]].
	^ String new! !

!RestRequestProcessor class methodsFor: 'instance creation' stamp: 'cs 2/23/2009 18:06'!
newFrom: anHTTPRequest handler: aHandler
	^ self new
		request: anHTTPRequest;
		handler: aHandler! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
handler

	^ handler! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
handler: anObject

	handler := anObject! !

!RestRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:18'!
process: root components: pathComponents
	"The root should be an association between name and class of the root resource."

	^ HTTPResponse ok entity:
		(self responseBodyForRoot: root components: pathComponents)! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
request

	^ request! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:58'!
request: anObject

	request := anObject! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'mt 3/2/2009 15:08'!
resolveId: aCollection components: pathComponents
	"Try to resolve the next element in the path to a number (id). This id should be a valid index in the given collection. Hint: Recursive."

	| id resource |
	pathComponents ifEmpty: [^ aCollection].
	
	id := pathComponents first asNumber.
	resource := aCollection 
		detect: [:each | each id = id] 
		ifNone: [RestResolveError signal: ('Entity with id {1} not found.' format: {id})].

	^ self resolveMethod: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'mt 3/2/2009 15:07'!
resolveMethod: anEntity components: pathComponents
	"Try to resolve the next element in the path to a method. This method/message should be understood by the entity. Hint: Recursive."
	
	| selector resource |
	pathComponents ifEmpty: [^ anEntity].
	
	selector := pathComponents first asSymbol.
	(anEntity class allSelectors includes: selector)
		ifTrue: [resource := anEntity perform: selector]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].

	^ self resolveId: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'helping' stamp: 'mt 3/2/2009 15:33'!
resolveUri: aString
	"Finds an object with the provided uri by using an pseudo-get-request. This is the only reason why we need a reference to the handler."

	| newRequest |	
	newRequest := HTTPGet request: aString.
	^ (RestRequestProcessor newFrom: newRequest handler: self handler)
		resolveId: (self handler
			rootComponent: newRequest
			ifAbsent: [RestResolveError signal: 'invalid URI given']) value all
		components: (self handler pathComponents: newRequest) allButFirst! !

!RestRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:14'!
responseBodyForRoot: root components: pathComponents

	self subclassResponsibility.! !

!Collection methodsFor: '*rest-server' stamp: 'topa 2/25/2009 13:25'!
jsonWriteOn: aStream

	aStream nextPut: $[.
	self
		do: [ :each | each jsonWriteOn: aStream]
		separatedBy: [aStream nextPut: $,].
	aStream nextPut: $].! !

Resource subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Services'!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService class methodsFor: 'testing' stamp: 'topa 3/3/2009 21:11'!
isRunning

	^ self site notNil and: [site isServing]! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:11'!
port

	^ self site port! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
resource

	^ resource! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
resource: anObject

	resource := anObject.! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
site

	^ site! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:13'!
site: anObject

	site := anObject.! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:08'!
start

	self startOn: 8082.

! !

!RestHttpService class methodsFor: 'service' stamp: 'topa 3/3/2009 21:14'!
startOn: aPort
	"Start exactly one http service. Does nothing if server is already running."

	self isRunning ifTrue: [^ self].
	self site: (Site new
		name: 'rest';
		host: '*' port: aPort).

	SwazooServer singleton addSite: self site.
	self resource: (RestHttpService new uriPattern: '/').
	self site addResource: self resource.
	SwazooServer singleton startSite: 'rest'.! !

!RestHttpService class methodsFor: 'service' stamp: 'topa 3/3/2009 21:15'!
stop
	"Shutdown the server. Does nothing if server is not running."

	self site ifNotNil:
		[self site stop.
		SwazooServer singleton removeSite: self site.
		self site: nil; resource: nil].! !

!RestHttpService methodsFor: 'serving' stamp: 'topa 2/23/2009 17:48'!
answerTo: request
	
	^ (handlers
		detect: [:ass| request uri asString startsWith: ass key]
		ifNone: [^ HTTPResponse notFound])
			value new process: request! !

!RestHttpService methodsFor: 'initialization' stamp: 'topa 3/3/2009 21:10'!
initialize

	super initialize.
	handlers := OrderedCollection new.	"needed for priorities"! !

!RestHttpService methodsFor: 'serving' stamp: 'topa 3/3/2009 21:09'!
installHandler: aRequestHandlerClass on: aPathString

	(handlers anySatisfy: [:ass | ass key = aPathString ]) ifTrue: [^ false].
	handlers add: aPathString -> aRequestHandlerClass.! !

Error subclass: #RestResolveError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResolveError methodsFor: 'as yet unclassified' stamp: 'topa 3/3/2009 21:02'!
asHTTPResponse

	^ HTTPResponse notFound entity: self messageText! !
